//+------------------------------------------------------------------+
//|                                                    SignalADX.mqh |
//|                             Copyright 2000-2024, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals of customized ADX oscillator.                      |
//| Type=SignalAdvanced                                              |
//| Name=ADX                                                         |
//| ShortName=ADX                                                    |
//| Class=CSignalADX                                                 |
//| Page=signal_adx                                                  |
//| Parameter=PeriodMA,int,14,Period of averaging                    |
//| Parameter=Applied,ENUM_APPLIED_PRICE,PRICE_CLOSE,Prices series   |
//| Parameter=AlternateFrame,ENUM_TIMEFRAMES,PERIOD_W1,Alternate TimeFrame |
//| Parameter=Pattern_0,int,50,Pattern 0                             |
//| Parameter=Pattern_1,int,50,Pattern 1                             |
//| Parameter=Pattern_2,int,50,Pattern 2                             |
//| Parameter=Pattern_3,int,50,Pattern 3                             |
//| Parameter=Pattern_4,int,50,Pattern 4                             |
//| Parameter=Pattern_5,int,50,Pattern 5                             |
//| Parameter=Pattern_6,int,50,Pattern 6                             |
//| Parameter=Pattern_7,int,50,Pattern 7                             |
//| Parameter=Pattern_8,int,50,Pattern 8                             |
//| Parameter=PatternsUsed,int,255,Patterns Used BitMap              |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSignalADX.                                                |
//| Purpose: Class of generator of trade signals based on            |
//|          customized ADX oscillator.                              |
//| Is derived from the CExpertSignal class.                         |
//+------------------------------------------------------------------+
#define  __COMPRESSION_LIMIT 65
class CSignalADX : public CExpertSignal
{
protected:
   CiADX             m_adx;            // object-oscillator
   CiADX             m_adx_alt;        // object-oscillator alternate
   CiMA              m_ma;             // object-indicator
   CiHigh            m_high_alt;
   CiLow             m_low_alt;
   CiRSI             m_rsi;
   CiATR             m_atr;
   //--- adjusted parameters
   int               m_ma_period;      // the "period of averaging" parameter of the oscillator
   ENUM_APPLIED_PRICE m_ma_applied;    // the "object of averaging" parameter of the oscillator
   ENUM_TIMEFRAMES   m_alternate;      // the "alternate time frame" parameter of the oscillator
   //--- "weights" of market models (0-100)
   int               m_pattern_0;      // model 0 "ADX Crossover with DI+ and DI-"
   int               m_pattern_1;      // model 1 "ADX Above 25 - Strong Trend Entry"
   int               m_pattern_2;      // model 2 "ADX Falling Below 25 - Trend Weakness"
   int               m_pattern_3;      // model 3 "ADX Crossover with Moving Average"
   int               m_pattern_4;      // model 4 "ADX and Price Divergence"
   int               m_pattern_5;      // model 5 "ADX and Range Breakout"
   int               m_pattern_6;      // model 6 "ADX Reversal Pattern"
   int               m_pattern_7;      // model 7 "ADX and Support/Resistance Break"
   int               m_pattern_8;      // model 8 "ADX with RSI Confirmation"

public:
                     CSignalADX(void);
                    ~CSignalADX(void);
   //--- methods of setting adjustable parameters
   void              PeriodMA(int value)
   {  m_ma_period = value;
   }
   void              Applied(ENUM_APPLIED_PRICE value)
   {  m_ma_applied = value;
   }
   void              AlternateFrame(ENUM_TIMEFRAMES value)
   {  m_alternate = value;
   }
   //--- methods of adjusting "weights" of market models
   void              Pattern_0(int value)
   {  m_pattern_0 = value;
   }
   void              Pattern_1(int value)
   {  m_pattern_1 = value;
   }
   void              Pattern_2(int value)
   {  m_pattern_2 = value;
   }
   void              Pattern_3(int value)
   {  m_pattern_3 = value;
   }
   void              Pattern_4(int value)
   {  m_pattern_4 = value;
   }
   void              Pattern_5(int value)
   {  m_pattern_5 = value;
   }
   void              Pattern_6(int value)
   {  m_pattern_6 = value;
   }
   void              Pattern_7(int value)
   {  m_pattern_7 = value;
   }
   void              Pattern_8(int value)
   {  m_pattern_8 = value;
   }
   void              PatternsUsed(int value)
   {  m_patterns_usage = value;
      PatternsUsage(value);
   }
   //--- method of verification of settings
   virtual bool      ValidationSettings(void);
   //--- method of creating the oscillator and timeseries
   virtual bool      InitIndicators(CIndicators *indicators);
   //--- methods of checking if the market models are formed
   virtual int       LongCondition(void);
   virtual int       ShortCondition(void);

protected:
   //--- method of initialization of the oscillator
   bool              InitADX(CIndicators *indicators);
   //--- methods of getting data
   double            Alt(int ind)
   {                 m_adx_alt.Refresh(-1);
      return(m_adx_alt.Main(ind));
   }
   double            Base(int ind)
   {                 m_adx.Refresh(-1);
      return(m_adx.Main(ind));
   }
   double            BasePlus(int ind)
   {                 m_adx.Refresh(-1);
      return(m_adx.Plus(ind));
   }
   double            BaseMinus(int ind)
   {                 m_adx.Refresh(-1);
      return(m_adx.Minus(ind));
   }
   double            BaseAverage(int ind)
   {                 m_adx.Refresh(-1);
      vector _v;
      _v.CopyIndicatorBuffer(m_adx.Handle(), 0, ind, m_ma_period);
      return(_v.Mean());
   }
   double            ATR(int ind)
   {                 m_atr.Refresh(-1);
      return(m_atr.Main(ind));
   }
   double            ATRAverage(int ind)
   {                 m_atr.Refresh(-1);
      vector _v;
      _v.CopyIndicatorBuffer(m_atr.Handle(), 0, ind, m_ma_period);
      return(_v.Mean());
   }
   double            Close(int ind)
   {                 m_close.Refresh(-1);
      return(m_close.GetData(ind));
   }
   double            High(int ind)
   {                 m_high.Refresh(-1);
      return(m_high.GetData(ind));
   }
   double            Low(int ind)
   {                 m_low.Refresh(-1);
      return(m_low.GetData(ind));
   }
   double            HighAlt(int ind)
   {                 m_high_alt.Refresh(-1);
      return(m_high_alt.GetData(ind));
   }
   double            LowAlt(int ind)
   {                 m_low_alt.Refresh(-1);
      return(m_low_alt.GetData(ind));
   }
   double            Range(int ind)
   {                 return(High(ind) - Low(ind));
   }
   double            MA(int ind)
   {                 m_ma.Refresh(-1);
      return(m_ma.Main(ind));
   }
   double            RSI(int ind)
   {                 m_rsi.Refresh(-1);
      return(m_rsi.Main(ind));
   }
   int               X()
   {  return(StartIndex());
   }
   bool              Compression(ENUM_POSITION_TYPE T, double &Out)
   {  Out = 0.0;
      int _i = X() + 1, _c = 0;
      double _last = Base(X() + 1);
      double _first = 0.0;
      if
      (
         T == POSITION_TYPE_BUY &&
         Base(X()) < Low(X()) &&
         Base(_i) < Close(X()) &&
         Base(_i) > High(_i)
      )
      {  while(Base(_i) > High(_i) && _c < __COMPRESSION_LIMIT)
         {  _first = Base(_i);
            _i++;
            _c++;
         }
         if(_c > 0)
         {  Out = fabs(_first - _last) / _c;
            return(true);
         }
      }
      else if
      (
         T == POSITION_TYPE_SELL &&
         Base(X()) > High(X()) &&
         Base(_i) > Close(X()) &&
         Base(_i) < Low(_i)
      )
      {  while(Base(_i) < Low(_i) && _c < __COMPRESSION_LIMIT)
         {  _first = Base(_i);
            _i++;
            _c++;
         }
         if(_c > 0)
         {  Out = fabs(_first - _last) / _c;
            return(true);
         }
      }
      return(false);
   }
   bool              Channel(ENUM_POSITION_TYPE T)
   {  vector _max, _max_i;
      vector _min, _min_i;
      _max.Init(2);
      _max.Fill(High(0));
      _max_i.Init(2);
      _max_i.Fill(0.0);
      _min.Init(2);
      _min.Fill(Low(0));
      _min_i.Init(2);
      _min_i.Fill(0.0);
      for(int i = 0; i < m_ma_period; i++)
      {  if(High(i) > _max[0])
         {  _max[0] = High(i);
            _max_i[0] = i;
         }
         if(Low(i) < _min[0])
         {  _min[0] = Low(i);
            _min_i[0] = i;
         }
      }
      double _slope = (Close(0) - Close(m_ma_period - 1)) / m_ma_period;
      double _upper_scale = fabs(_slope);
      double _lower_scale = fabs(_slope);
      for(int i = 0; i < m_ma_period; i++)
      {  if(i == _max_i[0])
         {  continue;
         }
         else
         {  double _i_slope = (High(i) - _max[0]) / (i - _max_i[0]);
            if((_i_slope > 0.0 && _slope > 0.0) || (_i_slope < 0.0 && _slope < 0.0))
            {  if(fabs(_i_slope - _slope) < _upper_scale)
               {  _max[1] = High(i);
                  _max_i[1] = i;
               }
            }
         }
      }
      for(int i = 0; i < m_ma_period; i++)
      {  if(i == _min_i[0])
         {  continue;
         }
         else
         {  double _i_slope = (Low(i) - _min[0]) / (i - _min_i[0]);
            if((_i_slope > 0.0 && _slope > 0.0) || (_i_slope < 0.0 && _slope < 0.0))
            {  if(fabs(_i_slope - _slope) < _lower_scale)
               {  _min[1] = Low(i);
                  _min_i[1] = i;
               }
            }
         }
      }
      vector _projections;
      _projections.Init(4);
      _projections[0] = _max[0] + (_max_i[0] * _slope);
      _projections[1] = _min[0] + (_min_i[0] * _slope);
      _projections[2] = _max[1] + (_max_i[1] * _slope);
      _projections[3] = _min[1] + (_min_i[1] * _slope);
      if(T == POSITION_TYPE_BUY && Close(0) < Close(m_ma_period) && Close(0) < _projections.Mean())
      {  return(true);
      }
      else if(T == POSITION_TYPE_SELL && Close(0) > Close(m_ma_period) && Close(0) > _projections.Mean())
      {  return(true);
      }
      return(false);
   }
   //--- methods to check for patterns
   bool              IsPattern_0(ENUM_POSITION_TYPE T);
   bool              IsPattern_1(ENUM_POSITION_TYPE T);
   bool              IsPattern_2(ENUM_POSITION_TYPE T);
   bool              IsPattern_3(ENUM_POSITION_TYPE T);
   bool              IsPattern_4(ENUM_POSITION_TYPE T);
   bool              IsPattern_5(ENUM_POSITION_TYPE T);
   bool              IsPattern_6(ENUM_POSITION_TYPE T);
   bool              IsPattern_7(ENUM_POSITION_TYPE T);
   bool              IsPattern_8(ENUM_POSITION_TYPE T);
};
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CSignalADX::CSignalADX(void) : m_ma_period(14),
   m_ma_applied(PRICE_CLOSE),
   m_alternate(PERIOD_W1),
   m_pattern_0(50),
   m_pattern_1(50),
   m_pattern_2(50),
   m_pattern_3(50),
   m_pattern_4(50),
   m_pattern_5(50),
   m_pattern_6(50),
   m_pattern_7(50),
   m_pattern_8(50)
{
//--- initialization of protected data
   m_used_series = USE_SERIES_OPEN + USE_SERIES_HIGH + USE_SERIES_LOW + USE_SERIES_CLOSE;
   PatternsUsage(m_patterns_usage);
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CSignalADX::~CSignalADX(void)
{
}
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//+------------------------------------------------------------------+
bool CSignalADX::ValidationSettings(void)
{
//--- validation settings of additional filters
   if(!CExpertSignal::ValidationSettings())
      return(false);
//--- initial data checks
   if(m_ma_period <= 0)
   {  printf(__FUNCTION__ + ": period MA must be greater than 0");
      return(false);
   }
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//+------------------------------------------------------------------+
bool CSignalADX::InitIndicators(CIndicators *indicators)
{
//--- check pointer
   if(indicators == NULL)
      return(false);
//--- initialization of indicators and timeseries of additional filters
   if(!CExpertSignal::InitIndicators(indicators))
      return(false);
//--- create and initialize MA oscillator
   if(!InitADX(indicators))
      return(false);
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| Initialize MA indicators.                                        |
//+------------------------------------------------------------------+
bool CSignalADX::InitADX(CIndicators *indicators)
{
//--- check pointer
   if(indicators == NULL)
      return(false);
//--- add object to collection
   if(!indicators.Add(GetPointer(m_adx)))
   {  printf(__FUNCTION__ + ": error adding object");
      return(false);
   }
//--- initialize object
   if(!m_adx.Create(m_symbol.Name(), m_period, m_ma_period))
   {  printf(__FUNCTION__ + ": error initializing object");
      return(false);
   }
   if(!m_adx_alt.Create(m_symbol.Name(), m_alternate, m_ma_period))
   {  printf(__FUNCTION__ + ": error initializing object");
      return(false);
   }
   if(!m_ma.Create(m_symbol.Name(), m_period, m_ma_period, 0, MODE_SMA, m_ma_applied))
   {  printf(__FUNCTION__ + ": error initializing object");
      return(false);
   }
   if(!m_high_alt.Create(m_symbol.Name(), m_alternate))
   {  printf(__FUNCTION__ + ": error initializing object");
      return(false);
   }
   if(!m_low_alt.Create(m_symbol.Name(), m_alternate))
   {  printf(__FUNCTION__ + ": error initializing object");
      return(false);
   }
   if(!m_rsi.Create(m_symbol.Name(), m_period, m_ma_period, m_ma_applied))
   {  printf(__FUNCTION__ + ": error initializing object");
      return(false);
   }
   if(!m_atr.Create(m_symbol.Name(), m_period, m_ma_period))
   {  printf(__FUNCTION__ + ": error initializing object");
      return(false);
   }
//--- ok
   return(true);
}
//+------------------------------------------------------------------+
//| "Voting" that price will grow.                                   |
//+------------------------------------------------------------------+
int CSignalADX::LongCondition(void)
{  int result  = 0, results = 0;
//--- if the model 0 is used and "ADX Crossover with DI+ and DI-"
   if(((m_patterns_usage & 0x01) != 0) && IsPattern_0(POSITION_TYPE_BUY))
   {  result += m_pattern_0;
      results++;
   }
//--- if the model 1 is used and "ADX Compression Zone"
   if(((m_patterns_usage & 0x02) != 0) && IsPattern_1(POSITION_TYPE_BUY))
   {  result += m_pattern_1;
      results++;
   }
//--- if the model 2 is used and "ADX Falling Below 25 - Trend Weakness"
   if(((m_patterns_usage & 0x04) != 0) && IsPattern_2(POSITION_TYPE_BUY))
   {  result += m_pattern_2;
      results++;
   }
//--- if the model 3 is used and "ADX Crossover with Moving Average"
   if(((m_patterns_usage & 0x08) != 0) && IsPattern_3(POSITION_TYPE_BUY))
   {  result += m_pattern_3;
      results++;
   }
//--- if the model 5 is used and "ADX Above 25 - Strong Trend Entry"
   if(((m_patterns_usage & 0x10) != 0) && IsPattern_4(POSITION_TYPE_BUY))
   {  result += m_pattern_4;
      results++;
   }
//--- if the model 5 is used and "ADX and Range Breakout"
   if(((m_patterns_usage & 0x20) != 0) && IsPattern_5(POSITION_TYPE_BUY))
   {  result += m_pattern_5;
      results++;
   }
//--- if the model 6 is used and "ADX Reversal Pattern"
   if(((m_patterns_usage & 0x40) != 0) && IsPattern_6(POSITION_TYPE_BUY))
   {  result += m_pattern_6;
      results++;
   }
//--- if the model 7 is used and "ADX and Support/Resistance Break"
   if(((m_patterns_usage & 0x80) != 0) && IsPattern_7(POSITION_TYPE_BUY))
   {  result += m_pattern_7;
      results++;
   }
//--- if the model 8 is used and "ADX with RSI Confirmation"
   if(((m_patterns_usage & 0x100) != 0) && IsPattern_8(POSITION_TYPE_BUY))
   {  result += m_pattern_8;
      results++;
   }
//--- return the result
//if(result > 0)printf(__FUNCSIG__+" result is: %i",result);
   if(results > 0)
   {  return(int(round(result / results)));
   }
   return(0);
}
//+------------------------------------------------------------------+
//| "Voting" that price will fall.                                   |
//+------------------------------------------------------------------+
int CSignalADX::ShortCondition(void)
{  int result  = 0, results = 0;
//--- if the model 0 is used and "ADX Crossover with DI+ and DI-"
   if(((m_patterns_usage & 0x01) != 0) && IsPattern_0(POSITION_TYPE_SELL))
   {  result += m_pattern_0;
      results++;
   }
//--- if the model 1 is used and "ADX Compression Zone"
   if(((m_patterns_usage & 0x02) != 0) && IsPattern_1(POSITION_TYPE_SELL))
   {  result += m_pattern_1;
      results++;
   }
//--- if the model 2 is used and "ADX Falling Below 25 - Trend Weakness"
   if(((m_patterns_usage & 0x04) != 0) && IsPattern_2(POSITION_TYPE_SELL))
   {  result += m_pattern_2;
      results++;
   }
//--- if the model 3 is used and "ADX Crossover with Moving Average"
   if(((m_patterns_usage & 0x08) != 0) && IsPattern_3(POSITION_TYPE_SELL))
   {  result += m_pattern_3;
      results++;
   }
//--- if the model 5 is used and "ADX Above 25 - Strong Trend Entry"
   if(((m_patterns_usage & 0x10) != 0) && IsPattern_4(POSITION_TYPE_SELL))
   {  result += m_pattern_4;
      results++;
   }
//--- if the model 5 is used and "ADX and Range Breakout"
   if(((m_patterns_usage & 0x20) != 0) && IsPattern_5(POSITION_TYPE_SELL))
   {  result += m_pattern_5;
      results++;
   }
//--- if the model 6 is used and "ADX Reversal Pattern"
   if(((m_patterns_usage & 0x40) != 0) && IsPattern_6(POSITION_TYPE_SELL))
   {  result += m_pattern_6;
      results++;
   }
//--- if the model 7 is used and "ADX and Support/Resistance Break"
   if(((m_patterns_usage & 0x80) != 0) && IsPattern_7(POSITION_TYPE_SELL))
   {  result += m_pattern_7;
      results++;
   }
//--- if the model 8 is used and "ADX with RSI Confirmation"
   if(((m_patterns_usage & 0x100) != 0) && IsPattern_8(POSITION_TYPE_SELL))
   {  result += m_pattern_8;
      results++;
   }
//--- return the result
//if(result > 0)printf(__FUNCSIG__+" result is: %i",result);
   if(results > 0)
   {  return(int(round(result / results)));
   }
   return(0);
}
//+------------------------------------------------------------------+
//| Check for Pattern 0.                                             |
//+------------------------------------------------------------------+
bool CSignalADX::IsPattern_0(ENUM_POSITION_TYPE T)
{  if(T == POSITION_TYPE_BUY && BasePlus(X() + 1) < BaseMinus(X() + 1) && BasePlus(X()) > BaseMinus(X()))
   {  return(true);
   }
   else if(T == POSITION_TYPE_SELL && BasePlus(X() + 1) > BaseMinus(X() + 1) && BasePlus(X()) < BaseMinus(X()))
   {  return(true);
   }
   return(false);
}
//+------------------------------------------------------------------+
//| Check for Pattern 1.                                             |
//+------------------------------------------------------------------+
bool CSignalADX::IsPattern_1(ENUM_POSITION_TYPE T)
{  if(T == POSITION_TYPE_BUY && Base(X()) > 25.0 && BasePlus(X()) > BaseMinus(X()))
   {  return(true);
   }
   else if(T == POSITION_TYPE_SELL && Base(X()) > 25.0 && BasePlus(X()) < BaseMinus(X()))
   {  return(true);
   }
   return(false);
}
//+------------------------------------------------------------------+
//| Check for Pattern 2.                                             |
//+------------------------------------------------------------------+
bool CSignalADX::IsPattern_2(ENUM_POSITION_TYPE T)
{  if(T == POSITION_TYPE_BUY && Base(X()+1) > 25.0 && Base(X()) < 25.0 && BasePlus(X()) < BaseMinus(X()))
   {  return(true);
   }
   else if(T == POSITION_TYPE_SELL && Base(X()+1) > 25.0 && Base(X()) < 25.0 && BasePlus(X()) > BaseMinus(X()))
   {  return(true);
   }
   return(false);
}
//+------------------------------------------------------------------+
//| Check for Pattern 3.                                             |
//+------------------------------------------------------------------+
bool CSignalADX::IsPattern_3(ENUM_POSITION_TYPE T)
{  if(T == POSITION_TYPE_BUY && Base(X()+1) < BaseAverage(X()+1) && Base(X()) > BaseAverage(X()) && BasePlus(X()) > BaseMinus(X()))
   {  return(true);
   }
   else if(T == POSITION_TYPE_SELL && Base(X()+1) < BaseAverage(X()+1) && Base(X()) > BaseAverage(X()) && BasePlus(X()) < BaseMinus(X()))
   {  return(true);
   }
   return(false);
}
//+------------------------------------------------------------------+
//| Check for Pattern 4.                                             |
//+------------------------------------------------------------------+
bool CSignalADX::IsPattern_4(ENUM_POSITION_TYPE T)
{  if(T == POSITION_TYPE_BUY && Close(X()+2) > Close(X()+1) && Close(X()+1) > Close(X()) && Base(X()+2) > Base(X()+1) && Base(X()+1) > Base(X()) && Base(X()) <= 25)
   {  return(true);
   }
   else if(T == POSITION_TYPE_SELL && Close(X()+2) < Close(X()+1) && Close(X()+1) < Close(X()) && Base(X()+2) > Base(X()+1) && Base(X()+1) > Base(X()) && Base(X()) <= 25)
   {  return(true);
   }
   return(false);
}
//+------------------------------------------------------------------+
//| Check for Pattern 5.                                             |
//+------------------------------------------------------------------+
bool CSignalADX::IsPattern_5(ENUM_POSITION_TYPE T)
{  if(T == POSITION_TYPE_BUY && ATRAverage(X()+1) > ATR(X()+1) && ATRAverage(X()) < ATR(X()) && ATR(X()) < High(X())-Low(X()) && Base(X()+1) < 25 && Base(X()) > 25 && Close(X()+1) < Close(X()))
   {  return(true);
   }
   else if(T == POSITION_TYPE_SELL && ATRAverage(X()+1) > ATR(X()+1) && ATRAverage(X()) < ATR(X()) && ATR(X()) < High(X())-Low(X()) && Base(X()+1) < 25 && Base(X()) > 25 && Close(X()+1) > Close(X()))
   {  return(true);
   }
   return(false);
}
//+------------------------------------------------------------------+
//| Check for Pattern 6.                                             |
//+------------------------------------------------------------------+
bool CSignalADX::IsPattern_6(ENUM_POSITION_TYPE T)
{  return(false);
}
//+------------------------------------------------------------------+
//| Check for Pattern 7.                                             |
//+------------------------------------------------------------------+
bool CSignalADX::IsPattern_7(ENUM_POSITION_TYPE T)
{  return(false);
}
//+------------------------------------------------------------------+
//| Check for Pattern 8.                                             |
//+------------------------------------------------------------------+
bool CSignalADX::IsPattern_8(ENUM_POSITION_TYPE T)
{  return(false);
}
//+------------------------------------------------------------------+
